FUNCTION Ruler( nRowAdj, nColAdj )
*=============================================================================*
*
*       RULER():  On-Screen row/column ruler.
*                 Can be called at anytime by assigning to a key with SET KEY.
*                 Great tool for coding complicated @..SAY..GET screens.
*
*    Programmer:  Craig Yellick                 Source  BEB817
*    (original)   Yellick Computing
*                 7 West Fourth Street
*                 St. Paul, Minnesota 55102
*                 (612) 291-7594
*
*  Placed in the public domain with no restrictions on August 26, 1987.
*  This function first appeared in the Oct '87 issue of Reference(Clipper).
*
*=============================================================================*
*
*  Usage:  Add the line "Set Key nnn TO Ruler" to your main procedure, where
*          nnn is the INKEY() value of the key to use to invoke the ruler.
*          For example, use the value -20 to assign the ruler to CTRL-F1.
*          Make sure the OBJ version of this file is linked into your
*          application.
*
*          The following keys are used to control the ruler-
*
*              Up Down Left Right  -  Take a wild guess
*              Home End            -  Upper left and lower right of screen
*              Space-Bar           -  Return to center of screen
*              Plus (+)            -  Toggle cursor type
*              Tab                 -  Toggle offset on/off
*              1..9                -  Movement multiplier for rapid positioning
*                                     (Primarily for column movement, row
*                                     movement will stay somewhat proportional)
*              Esc                 -  Exit
*
*          Line zero will contain a status display:
*
*                              Ruler @ r,c  Move xN
*
*          Where r = row, c = column and N is the movement multiplier.
*          If the offset toggle is on an additional phrase is displayed:
*
*                                   Offset x,y
*
*          Where x = rows, x = columns relative to the main cursor position.
*          This is useful for calculating relative coordinates for boxes.
*          The offset ruler is turned on and off with the TAB key.
*
*  Notes:  I have used BORING black & white SET COLOR TO commands so this
*          procedure will work well on all monitors.  For best results and
*          a stunning effect, try changing the color of the various cursor
*          types and lines.  If you have access to a routine that changes the
*          background color of a screen area without disturbing the characters
*          you could make some great improvements as well.
*
*=============================================================================*
* Changes by John Wright
* 02/14/90 - Added ProClip CURSOR calls and save screen colors.
*            Put a call to HELP in order to remember commands.
* 06/15/92 - Converted to Clipper 5.01 for addition to STRIDE5 library.
* 03/09/93 - Added optional adjustment parameters for mainframe screens.

*
* Don't want to disturb the rest of the system...
*
LOCAL cr, cc, R, k, redraw, crosshairs, offset, scr_ruler, xcur, ycur
LOCAL rinit, cinit, rmax, cmax, rmin, cmin, rmult, cmult, rcur, ccur
LOCAL cOldColor, cSaveScrn, Getlist := {}

* Optional adjustments for mainframe screens.  Usually the row and col
* are off by one compared to Clipper.  Row/Col starts at 1/1 not 0/0.
nRowAdj := IF( VALTYPE( nRowAdj ) <> "N", 0, nRowAdj )
nColAdj := IF( VALTYPE( nColAdj ) <> "N", 0, nColAdj )

cr := ROW()  // Save current cursor row and column so we
cc := COL()  // can restore the screen exactly as we found it

cOldColor := SETCOLOR("W/N,N/W,N")    // save current system colors
cSaveScrn := Savebox(0,0,Maxrow(),Maxcol() )

*
* Define Constants  (These make the code easier to read)
*
rmin  :=  0                 // Minimum row
cmin  :=  0                 // Minimum column
rmax  :=  MAXROW()          // Maximum row
cmax  :=  MAXCOL()          // Maximum column
rinit := INT(rmax/2)        // Initial row
cinit := INT(cmax/2)        // Initial column

*
* Initialize Variables
*
rcur   :=     rinit       // Current row
ccur   :=     cinit       // Current column
xcur   :=     rinit       // Current offset row
ycur   :=     cinit       // Current offset column
rmult  :=     1           // Row movement multiplier
cmult  :=     1           // Column movement multiplier
redraw :=     .T.         // Screen redraw flag
offset :=     .F.         // Offset cursor toggle
crosshairs := .T.         // Cross hairs/block display toggle

*
*  Let's Do It To It
*
DO WHILE .T.  // We'll in stay here until ESC key is pressed
 
 *
 *  Screen Redraw
 *
 IF redraw
  redraw := .F.
  Restbox( cSaveScrn, "Quick" )  // Bring back the original screen
  
  @ 0,0                          // Clear the status line and redisplay
  @ 0,0 SAY "Ruler @"
  @ 0, COL()+1 SAY LTRIM(STR(rcur+nRowAdj, 2)) +","
  @ 0, COL()+0 SAY LTRIM(STR(ccur+nColAdj, 2))
  IF offset
   @ 0, COL()+2 SAY "Offset"
   @ 0, COL()+1 SAY LTRIM(STR(xcur -rcur, 3)) +","
   @ 0, COL()+0 SAY LTRIM(STR(ycur -ccur, 3))
  ENDIF
  @ 0, COL()+2 SAY "Move x"
  @ 0, COL()+0 SAY LTRIM(STR(cmult,2))
  
  *
  *  Display either cross-hair cursor or a flashing block
  *
  IF crosshairs
   DO CASE
    
    *  We're in the first column
   CASE ccur = cmin
    @ rcur, cmin +1 SAY REPLICATE(CHR(250), cmax -1) +CHR(180)

    *  We're in the last column
   CASE ccur = cmax
    @ rcur, cmin SAY CHR(195) +REPLICATE(CHR(250), cmax -1)

    *  We're somewhere in between
   OTHERWISE
    @ rcur, cmin SAY CHR(195) +REPLICATE(CHR(250), ccur -1)
    @ rcur, ccur +1 SAY REPLICATE(CHR(250), cmax -(ccur +1)) +CHR(180)
   ENDCASE
   
   *  If we're not on the first row
   IF rcur <> rmin
    @ rmin, ccur SAY CHR(194)
   ENDIF
   
   *  Do the middle of the line
   R= rmin +1
   DO WHILE R < rmax
    @ R, ccur SAY IIF(R = rcur, [], CHR(250))   // Skip our current pos
    R= R +1
   ENDDO
   
   *  If we're not on the last row
   IF rcur <> rmax
    @ rmax, ccur SAY CHR(193)
   ENDIF

  ELSE                        // Flashing block, instead
   @ rcur, ccur SAY CHR(254) COLOR "W*/N"
   
  ENDIF // Drawing the cross-hairs
  
  *
  *  Display offset cursor if needed
  *
  IF offset
     @ xcur, ycur SAY CHR(197) COLOR "W*/N"
  ENDIF

 ENDIF // Redraw
 
 k := INKEY(0)   //  Get a keypress
 
 redraw= .T.  //  Assume we'll have to redraw the screen
 
 *
 *  Interpret the key
 *
 DO CASE
 CASE k == 28                     // [F1] pressed
  DO rulehelp
 CASE k == 27                     // Escape
  EXIT
 CASE k == 43                     // (+) Toggle cross hairs
  crosshairs := !crosshairs
 CASE k ==  9                     // (Tab) Toggle offset cursor
  offset= .NOT. offset
 CASE k ==  5                     // Row Up
  IF offset
   xcur := xcur -rmult
  ELSE
   rcur := rcur -rmult
  ENDIF
 CASE k == 24                     // Row Down
  IF offset
   xcur := xcur +rmult
  ELSE
   rcur := rcur +rmult
  ENDIF
 CASE k == 19                     // Column Left
  IF offset
   ycur := ycur -cmult
  ELSE
   ccur := ccur -cmult
  ENDIF
 CASE k ==  4                     // Column Right
  IF offset
   ycur := ycur +cmult
  ELSE
   ccur := ccur +cmult
  ENDIF
 CASE k ==  32                    // (Space Bar) Center of Screen
  IF offset
   xcur := rinit
   ycur := cinit
  ELSE
   rcur := rinit
   ccur := cinit
  ENDIF
 CASE k ==  1                     // (Home) Upper Left Corner
  IF offset
   xcur := rmin
   ycur := cmin
  ELSE
   rcur := rmin
   ccur := cmin
  ENDIF
 CASE k ==  6                     // (End) Lower Right Corner
  IF offset
   xcur := rmax
   ycur := cmax
  ELSE
   rcur := rmax
   ccur := cmax
  ENDIF
 CASE (k > 48) .AND. (k < 58)    // (Keys 1..9) Move Multiplier
  cmult := k -48
  *
  *  Make row movement proportional to column
  *  but be sure it stays at least one
  *
  rmult := MAX(1, ROUND(cmult *(rmax/cmax),0) )
  
 OTHERWISE
  redraw := .F.   //  Keypress did not require screen redraw
  
 ENDCASE  // Interpreting keypress
 
 *
 * Perform row/col range checking, wrap around if necessary
 *
 rcur := WRAP(rcur, rmin, rmax)
 ccur := WRAP(ccur, cmin, cmax)
 xcur := WRAP(xcur, rmin, rmax)
 ycur := WRAP(ycur, cmin, cmax)
 
ENDDO  // While Escape not pressed

SETCOLOR(cOldColor)                // reset screen colors
Restbox( cSaveScrn, "quick" )      // All done, restore original screen,
@ cr, cc SAY []                    // and parked it back where we found it

RETURN .T.


*!*********************************************************************
*!
*!       Function: WRAP()
*!
*!*********************************************************************
STATIC FUNCTION WRAP( cur, MIN, MAX )
* Wrap cursor postion between min and max values
RETURN IIF(cur > MAX, MIN, IIF(cur < MIN, MAX, cur))


*!*********************************************************************
*!
*!      Procedure: RULEHELP
*!
*!*********************************************************************
STATIC FUNCTION rulehelp()
LOCAL rhelp
rhelp := SETCOLOR("N/W")
CLEAR SCREEN
TEXT
           [ ] [ ] [ ] [ ]  -  Take a wild guess
           [Home] [End]     -  Upper left and lower right of screen
           [Space bar]      -  Return to center of screen
           [+] plus         -  Toggle cursor type
           [Tab]            -  Toggle offset on/off
           [1]..[9]         -  Movement multiplier for rapid positioning
                               (Primarily for column movement, row
                                movement will stay somewhat proportional)
           [Esc]            -  Exit

           Line zero will contain a status display:

                               Ruler @ r,c  Move xN

           Where r = row, c = column and N is the movement multiplier.
           If the offset toggle is on an additional phrase is displayed:

                                    Offset x,y

           Where x = rows, x = columns relative to the main cursor position.
           This is useful for calculating relative coordinates for boxes.
           The offset ruler is turned on and off with the TAB key.
ENDTEXT
@ 1,12 SAY CHR(24)    // up arrow
@ 1,16 SAY CHR(25)    // down arrow
@ 1,20 SAY CHR(27)    // left arrow
@ 1,24 SAY CHR(26)    // right arrow
INKEY(0)
SETCOLOR(rhelp)
RETURN .T.

*: EOF: RULER.PRG
